@@ -4,8 +4,10 @@
     begin                : Mon Feb 12 23:45:41 EST 2001
     copyright            : (C) 2001-2003 by Sarang Lakare
     email                : sarang#users.sourceforge.net
-    copyright            : (C) 2003-2005 by Olaf Schmidt
+    copyright            : (C) 2003-2004 by Olaf Schmidt
     email                : ojschmidt@kde.org
+    copyright            : (C) 2008 by Matthew Woehlke
+    email                : mw_triad@users.sourceforge.net
  ***************************************************************************/
 
 /***************************************************************************
@@ -21,16 +23,22 @@
 // application specific includes
 #include "kmagzoomview.h"
 #include "kmagzoomview.moc"
+#include "colorsim.h"
 
 // include files for Qt
-#include <qbitmap.h>
-#include <qpixmap.h>
-#include <qimage.h>
-#include <qcursor.h>
-#include <qglobal.h>
-#include <qpainter.h>
-#include <qwhatsthis.h>
-#include <qwidget.h>
+#include <QtGui/QBitmap>
+#include <QtGui/QCursor>
+#include <QtCore/qglobal.h>
+#include <QtGui/QPainter>
+#include <QtGui/QWidget>
+#include <QtGui/QPixmap>
+#include <QtGui/QFocusEvent>
+#include <QtGui/QHideEvent>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QShowEvent>
+#include <QtGui/QResizeEvent>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QDesktopWidget>
 
 // include files for KDE
 #include <kapplication.h>
@@ -38,9 +46,6 @@
 #include <kdebug.h>
 #include <klocale.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-
 // include bitmaps for cursors
 static uchar left_ptr_bits[] = {
    0x00, 0x00, 0x08, 0x00, 0x18, 0x00, 0x38, 0x00, 0x78, 0x00, 0xf8, 0x00,
@@ -78,71 +83,33 @@ static uchar phand_bits[] = {
   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00 };
 
 
-
-static bool obscuredRegion (QRegion &region, Window winId, Window ignoreId, Window start = 0, int level = -1) {
-  Window root, parent, *children; uint nchildren;
-  if (0 == start)
-    start = qt_xrootwin();
-
-  bool winIdFound = false;
-  if (0 != XQueryTree (qt_xdisplay(), start, &root, &parent, &children, &nchildren)) {
-    for (uint i=0; i < nchildren; ++i) {
-      if (winIdFound) {
-        if (ignoreId != children [i]) {
-          XWindowAttributes atts;
-          XGetWindowAttributes (qt_xdisplay(), children [i], &atts);
-          if (atts.map_state == IsViewable)
-            region -= QRegion (atts.x, atts.y, atts.width, atts.height, QRegion::Rectangle);
-        }
-      }
-      else if (winId == children [i])
-        winIdFound = true;
-
-      // According to tests, my own window ID is either on toplevel or two levels below.
-      // To avoid unneccessary recursion, we limit the search to two recursion levels,
-      // then to five recursion levels, and make a full recursive search only if that
-      // was unsuccessful.
-      else if (level > 1)
-        winIdFound = obscuredRegion (region, winId, ignoreId, children [i], level-1);
-      else if (level == -1)
-        if (! (winIdFound = obscuredRegion (region, winId, ignoreId, children [i], 0)))
-          if (! (winIdFound = obscuredRegion (region, winId, ignoreId, children [i], 1)))
-            winIdFound = obscuredRegion (region, winId, ignoreId, children [i], -1);
-    }
-
-    if (children != NULL)
-      XFree (children);
-  }
-
-  return winIdFound;
-}
-
-
-
-
 KMagZoomView::KMagZoomView(QWidget *parent, const char *name)
-  : QScrollView(parent, name),
+  : Q3ScrollView(parent, name),
     m_selRect(0, 0, 128, 128, this),
-    m_grabTimer(0),
-    m_mouseViewTimer(0),
+    m_grabTimer(parent),
+    m_mouseViewTimer(parent),
     m_latestCursorPos(0,0),
     m_followMouse(false),
     m_showMouse(1),
     m_zoom(1.0),
     m_rotation(0),
-    m_invert(false),
     m_fitToWindow(true)
 {
-  KApplication::setGlobalMouseTracking(TRUE);
-  viewport()->setMouseTracking(TRUE);
-  viewport()->setBackgroundMode (NoBackground);
-  viewport()->setFocusPolicy(QWidget::StrongFocus);
-  
+  KApplication::setGlobalMouseTracking(true);
+  viewport()->setMouseTracking(true);
+  viewport()->setBackgroundMode (Qt::NoBackground);
+  viewport()->setFocusPolicy(Qt::StrongFocus);
+
+  // init the zoom matrix
+  m_zoomMatrix.reset();
+  m_zoomMatrix.scale(m_zoom, m_zoom);
+  m_zoomMatrix.rotate(m_rotation);
+
   m_ctrlKeyPressed = false;
   m_shiftKeyPressed = false;
   m_refreshSwitch = true;
   m_refreshSwitchStateOnHide = m_refreshSwitch;
-  
+
   // set the refresh rate
   setRefreshRate(10);
 
@@ -156,18 +123,19 @@ KMagZoomView::KMagZoomView(QWidget *parent, const char *name)
   // start the grabTimer @ 25 frames per second!
   m_mouseViewTimer.start(25);
 
-  QWhatsThis::add(this, i18n("This is the main window which shows the contents of the\
+  this->setWhatsThis( i18n("This is the main window which shows the contents of the\
  selected region. The contents will be magnified according to the zoom level that is set."));
 
   // different ways to show the cursor.
   m_showMouseTypes << "Hidden" << "Box" << "Arrow" << "Actual";
 
-  updateMatrix();
+  if(m_fitToWindow)
+    fitToWindow();
 }
 
 KMagZoomView::~KMagZoomView()
 {
-  KApplication::setGlobalMouseTracking(FALSE);
+  KApplication::setGlobalMouseTracking(true);
 }
 
 /**
@@ -178,13 +146,13 @@ void KMagZoomView::followMouse(bool follow)
   if(follow) {
     m_followMouse = true;
     m_mouseMode = Normal;
-    setVScrollBarMode (QScrollView::AlwaysOff);
-    setHScrollBarMode (QScrollView::AlwaysOff);
+    setVScrollBarMode (Q3ScrollView::AlwaysOff);
+    setHScrollBarMode (Q3ScrollView::AlwaysOff);
   } else {
     m_followMouse = false;
     m_mouseMode = Normal;
-    setVScrollBarMode (QScrollView::AlwaysOn);
-    setHScrollBarMode (QScrollView::AlwaysOn);
+    setVScrollBarMode (Q3ScrollView::AlwaysOn);
+    setHScrollBarMode (Q3ScrollView::AlwaysOn);
   }
 }
 
@@ -221,71 +189,85 @@ void KMagZoomView::showEvent( QShowEvent* )
  */
 void KMagZoomView::resizeEvent( QResizeEvent * e )
 {
-  QScrollView::resizeEvent (e);
+  Q3ScrollView::resizeEvent (e);
   if(m_fitToWindow)
     fitToWindow();
 }
 
 /**
- * Called when the widget is to be repainted
+ * Called when the widget is to be repainted.
  *
  * @param p
  */
 void KMagZoomView::drawContents ( QPainter * p, int clipx, int clipy, int clipw, int cliph )
 {
-  if(m_grabbedPixmap.isNull())
+  if(m_zoomedPixmap.isNull())
     return;
 
+  // Paint empty areas Qt::black
+  if (contentsX()+contentsWidth() < visibleWidth())
+    p->fillRect (
+        QRect (contentsX()+contentsWidth(), clipy, visibleWidth()-contentsX()-contentsWidth(), cliph)
+        & QRect (clipx, clipy, clipw, cliph),
+        Qt::black);
+  if (contentsY()+contentsHeight() < visibleHeight())
+    p->fillRect (
+        QRect (clipx, contentsY()+contentsHeight(), clipw, visibleHeight()-contentsY()-contentsHeight())
+        & QRect (clipx, clipy, clipw, cliph),
+        Qt::black);
+
   // A pixmap which will be eventually displayed
-  QRect areaToPaint = m_invertedMatrix.mapRect (QRect(clipx, clipy, clipw, cliph));
-  QPixmap clippedPixmap (areaToPaint.size());
-  clippedPixmap.fill (QColor (128, 128, 128));
-  areaToPaint &= QRect (QPoint (0,0), m_selRect.size());
-  bitBlt(&clippedPixmap, QPoint (0,0), &m_grabbedPixmap, areaToPaint);
+  QPixmap *zoomView;
+
+  // Get mouse position relative to the image
+  QPoint mousePos = calcMousePos (m_refreshSwitch);
 
   // show the pixel under mouse cursor
   if(m_showMouse) {
+
+    // Pixmap which will have the zoomed pixmap + mouse
+    zoomView = new QPixmap(m_zoomedPixmap);
+
     // paint the mouse cursor
-    paintMouseCursor(&clippedPixmap, calcMousePos (m_refreshSwitch)-QPoint (areaToPaint.x(), areaToPaint.y()));
+    paintMouseCursor(zoomView, mousePos);
+  } else { // do not show mouse
+    zoomView = &m_zoomedPixmap;
   }
 
-  QPixmap zoomedPixmap;
-  zoomedPixmap = clippedPixmap.xForm (m_zoomMatrix);
+  // bitBlt this part on to the widget.
+  bitBlt(viewport(), QPoint (clipx-contentsX(), clipy-contentsY()), zoomView, QRect(clipx, clipy, clipw, cliph));
 
-  if (m_invert) {
-    QImage zoomedImage;
-    zoomedImage = zoomedPixmap.convertToImage();
-    zoomedImage.invertPixels (false);
-    p->drawImage (QPoint (clipx-contentsX(), clipy-contentsY()), zoomedImage, zoomedImage.rect(),
-                  Qt::ThresholdDither | Qt::ThresholdAlphaDither | Qt::AvoidDither);
-  } else {
-    p->drawPixmap (QPoint (clipx, clipy), zoomedPixmap, zoomedPixmap.rect());
-  }
+  if(zoomView != &m_zoomedPixmap)
+    delete zoomView;
 }
 
 /**
  * Draws the mouse cursor according to the current selection of the type of
  * mouse cursor to draw.
  */
-void KMagZoomView::paintMouseCursor(QPaintDevice *dev, QPoint mousePos)
+void KMagZoomView::paintMouseCursor(QPaintDevice *dev, const QPoint &mousePos)
 {
   if(!dev)
     return;
 
+  // painter for the zoom view
   QPainter pz(dev);
 
   if(m_latestCursorPos.x() >= 0 && m_latestCursorPos.x() < m_selRect.width() &&
-     m_latestCursorPos.y() >= 0 && m_latestCursorPos.y() < m_selRect.height() ) {
+     m_latestCursorPos.y() >= 0 && m_latestCursorPos.y() < m_selRect.height() ) { // || updateMousePos) {
     // mouse position is indeed inside the selRect
-      
+
     // How to show the mouse :
 
     switch(m_showMouse) {
     case 1:
       // 1. Square around the pixel
       pz.setPen(Qt::white);
-      pz.setRasterOp(Qt::XorROP);
-      pz.drawRect(mousePos.x()-1, mousePos.y()-1, 3, 3);
+#ifdef __GNUC__
+#warning "Port Qt4 pz.setRasterOp(Qt::XorROP);";
+#endif
+      //pz.setRasterOp(Qt::XorROP);
+      pz.drawRect(mousePos.x()-1, mousePos.y()-1, (int)m_zoom+2, (int)m_zoom+2);
       break;
 
     case 2:
@@ -294,41 +276,50 @@ void KMagZoomView::paintMouseCursor(QPaintDevice *dev, QPoint mousePos)
       pz.setPen(Qt::black);
       pz.setBackgroundColor(Qt::white);
 
-      QBitmap sCursor( 16, 16, left_ptr_bits, TRUE );
-      QBitmap mask( 16, 16, left_ptrmsk_bits, TRUE );
+      QBitmap sCursor = QBitmap::fromData( QSize(16,  16),  left_ptr_bits);
+      QBitmap mask = QBitmap::fromData( QSize(16,  16),  left_ptrmsk_bits);
       sCursor.setMask(mask);
+      sCursor = sCursor.transformed(m_zoomMatrix);
 
       // since hot spot is at 3,1
-      pz.drawPixmap(mousePos.x()-3, mousePos.y()-1, sCursor);
+      if (m_rotation == 0)
+        pz.drawPixmap(mousePos.x()-(int)(3.0*m_zoom), mousePos.y()-(int)m_zoom, sCursor);
+      else if (m_rotation == 90)
+        pz.drawPixmap(mousePos.x()-(int)(16.0*m_zoom), mousePos.y()-(int)(3.0*m_zoom), sCursor);
+      else if (m_rotation == 180)
+        pz.drawPixmap(mousePos.x()-(int)(13.0*m_zoom), mousePos.y()-(int)(16.0*m_zoom), sCursor);
+      else if (m_rotation == 270)
+        pz.drawPixmap(mousePos.x()-(int)m_zoom, mousePos.y()-(int)(13.0*m_zoom), sCursor);
     }
     break;
 
     case 3:
-    {    
+    {
       // 3. Actual cursor
       // Get the current cursor type
-      QWidget *dummy  = KApplication::widgetAt(QCursor::pos(), FALSE);
+      QWidget *dummy  = KApplication::topLevelAt(QCursor::pos());
       if(!dummy)
         break;
-      kdDebug() << ">" << dummy->name() << ":" << dummy->cursor().shape() << "-" << endl;
+      kDebug() << ">" << dummy->name() << ":" << dummy->cursor().shape() << "-";
       switch(this->cursor().shape())  {
-        case ArrowCursor :
+			  case Qt::ArrowCursor :
          {
           // 2. Arrow cursor
           pz.setPen(Qt::black);
           pz.setBackgroundColor(Qt::white);
 
-          QBitmap sCursor( 16, 16, left_ptr_bits, TRUE );
-          QBitmap mask( 16, 16, left_ptrmsk_bits, TRUE );
+          QBitmap sCursor = QBitmap::fromData( QSize(16,  16),  left_ptr_bits);
+          QBitmap mask = QBitmap::fromData( QSize(16,  16),  left_ptrmsk_bits);
           sCursor.setMask(mask);
+          sCursor = sCursor.transformed(m_zoomMatrix);
 
           // since hot spot is at 3,1
-          pz.drawPixmap(mousePos.x()-3, mousePos.y()-1, sCursor);
+          pz.drawPixmap(mousePos.x()-(int)(3.0*m_zoom), mousePos.y()-(int)m_zoom, sCursor);
         }
         break;
         default:
-          QBitmap sCursor( 32, 32, phand_bits, TRUE );
-          QBitmap mask( 32, 32, phandm_bits, TRUE );
+          QBitmap sCursor = QBitmap::fromData( QSize(32,  32),  phand_bits);
+          QBitmap mask = QBitmap::fromData( QSize(32,  32),  phandm_bits);
           sCursor.setMask(mask);
 
           pz.drawPixmap(mousePos.x(), mousePos.y(), sCursor);
@@ -355,22 +346,33 @@ QPoint KMagZoomView::calcMousePos(bool updateMousePos)
     m_latestCursorPos -= QPoint(m_selRect.x(), m_selRect.y());
   }
 
-  // get coordinates of the pixel w.r.t. the pixmap
-  return QPoint (m_latestCursorPos.x(), m_latestCursorPos.y());
+  // get coordinates of the pixel w.r.t. the zoomed pixmap
+  if (m_rotation == 90)
+    return QPoint ((int)((float)(m_selRect.height()-m_latestCursorPos.y())*m_zoom),
+                   (int)((float)m_latestCursorPos.x()*m_zoom));
+  else if (m_rotation == 180)
+    return QPoint ((int)((float)(m_selRect.width()-m_latestCursorPos.x())*m_zoom),
+                   (int)((float)(m_selRect.height()-m_latestCursorPos.y())*m_zoom));
+  else if (m_rotation == 270)
+    return QPoint ((int)((float)m_latestCursorPos.y()*m_zoom),
+                   (int)((float)(m_selRect.width()-m_latestCursorPos.x())*m_zoom));
+  else
+    return QPoint ((int)((float)m_latestCursorPos.x()*m_zoom),
+                   (int)((float)m_latestCursorPos.y()*m_zoom));
 }
 
 
 // MOUSE ACTIONS
 
 /**
- * Called when mouse is clicked inside the window
+ * Called when mouse is clicked inside the window.
  *
  * @param e
  */
 void KMagZoomView::mousePressEvent(QMouseEvent *e)
 {
   switch(e->button()) {
-  case QMouseEvent::LeftButton :
+  case Qt::LeftButton :
     if(m_ctrlKeyPressed) {
       // check if currently in resize mode
       // don't do anything if fitToWindow is enabled
@@ -379,7 +381,7 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
         m_mouseMode = ResizeSelection;
 
         // set mouse cursor to "resize all direction"
-        setCursor(sizeAllCursor);
+        setCursor(Qt::SizeAllCursor);
 
         // backup the old position
         m_oldMousePos.setX(e->globalX());
@@ -392,10 +394,8 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
         m_selRect.show();
       }
       else {
-        #if QT_VERSION >= 300
         // ignore this button press.. so it goes to the parent
         e->ignore();
-        #endif
       }
     } else if(m_shiftKeyPressed) {
       // check if currently in move mode
@@ -404,7 +404,7 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
         m_mouseMode = MoveSelection;
 
         // set mouse cursor to cross hair
-        setCursor(crossCursor);
+        setCursor(Qt::CrossCursor);
 
         // backup the old position
         m_oldMousePos.setX(e->globalX());
@@ -417,10 +417,8 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
         m_selRect.show();
       }
       else {
-        #if QT_VERSION >= 300
         // ignore this button press.. so it goes to the parent
         e->ignore();
-        #endif
       }
     } else {
       // check if currently in move mode
@@ -429,7 +427,7 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
         m_mouseMode = GrabSelection;
 
         // set mouse cursor to hand
-        setCursor(KCursor::handCursor());
+        setCursor(Qt::PointingHandCursor);
 
         // store the old position
         m_oldMousePos.setX(e->globalX());
@@ -441,22 +439,20 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
         m_selRect.show();
       }
       else {
-        #if QT_VERSION >= 300
         // ignore this button press.. so it goes to the parent
         e->ignore();
-        #endif
       }
     }
     break;
 
-  case QMouseEvent::MidButton :
+  case Qt::MidButton :
     // check if currently in move mode
     // don't do anything if follow mouse is enabled
     if ((m_mouseMode != MoveSelection) && !m_followMouse) {
       m_mouseMode = MoveSelection;
 
       // set mouse cursor to cross hair
-      setCursor(crossCursor);
+      setCursor(Qt::CrossCursor);
 
       // backup the old position
       m_oldMousePos.setX(e->globalX());
@@ -469,18 +465,14 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
       m_selRect.show();
     }
     else {
-      #if QT_VERSION >= 300
       // ignore this button press.. so it goes to the parent
       e->ignore();
-      #endif
     }
     break;
   // do nothing
   default:
-#if QT_VERSION >= 300
     // ignore this button press.. so it goes to the parent
     e->ignore();
-#endif
     break;
   }
 }
@@ -494,8 +486,8 @@ void KMagZoomView::mousePressEvent(QMouseEvent *e)
 void KMagZoomView::mouseReleaseEvent(QMouseEvent *e)
 {
   switch(e->button()) {
-  case QMouseEvent::LeftButton :
-  case QMouseEvent::MidButton :
+  case Qt::LeftButton :
+  case Qt::MidButton :
     // check if currently in move mode
     if(m_mouseMode == MoveSelection) {
       // hide the selection window
@@ -504,7 +496,7 @@ void KMagZoomView::mouseReleaseEvent(QMouseEvent *e)
       m_mouseMode = Normal;
 
       // restore the cursor shape
-      setCursor(arrowCursor);
+      setCursor(Qt::ArrowCursor);
 
       // restore the cursor position
       QCursor::setPos(m_oldMousePos);
@@ -515,7 +507,7 @@ void KMagZoomView::mouseReleaseEvent(QMouseEvent *e)
       m_mouseMode = Normal;
 
       // restore the cursor shape
-      setCursor(arrowCursor);
+      setCursor(Qt::ArrowCursor);
 
       // restore the cursor position
       QCursor::setPos(m_oldMousePos);
@@ -527,13 +519,13 @@ void KMagZoomView::mouseReleaseEvent(QMouseEvent *e)
       m_mouseMode = Normal;
 
       // restore the cursor shape
-      setCursor(arrowCursor);
-    }    
+      setCursor(Qt::ArrowCursor);
+    }
     break;
 
-  case QMouseEvent::RightButton :
+  case Qt::RightButton :
     break;
-  case QMouseEvent::NoButton :
+  case Qt::NoButton :
     break;
 
   // do nothing
@@ -556,25 +548,66 @@ void KMagZoomView::mouseMoveEvent(QMouseEvent *e)
     m_selRect.setRight(e->globalX());
     m_selRect.setBottom(e->globalY());
     m_selRect.update();
-
     grabFrame();
   } else if(m_mouseMode == MoveSelection) {
-    QPoint newCenter;
+     QPoint newCenter;
 
     // set new center to be the current mouse position
-    m_selRect.moveCenter(e->globalPos());
-    m_selRect.update();
+    newCenter = e->globalPos();
 
+    // make sure the mouse position is not taking the grab window outside
+    // the display
+    if(newCenter.x() < m_selRect.width()/2) {
+      // set X to the minimum possible X
+      newCenter.setX(m_selRect.width()/2);
+    } else if(newCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
+      // set X to the maximum possible X
+      newCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
+    }
+
+    if(newCenter.y() < m_selRect.height()/2) {
+      // set Y to the minimum possible Y
+      newCenter.setY(m_selRect.height()/2);
+    } else if(newCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
+      // set Y to the maximum possible Y
+      newCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
+    }
+    // move to the new center
+    m_selRect.moveCenter(newCenter);
+    // update the grab rectangle display
+    m_selRect.update();
     grabFrame();
   } else if(m_mouseMode == GrabSelection) {
-    QPoint newPos;
+     QPoint newPos;
 
     // get new position
     newPos = e->globalPos();
-    QPoint delta = (newPos - m_oldMousePos)/m_zoom;
-    m_selRect.moveCenter(m_oldCenter-delta);
-    m_selRect.update();
 
+    QPoint delta = (newPos - m_oldMousePos)/m_zoom;
+    QPoint newCenter = m_oldCenter-delta;
+
+    // make sure the mouse position is not taking the grab window outside
+    // the display
+    if(newCenter.x() < m_selRect.width()/2) {
+      // set X to the minimum possible X
+      newCenter.setX(m_selRect.width()/2);
+    } else if(newCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
+      // set X to the maximum possible X
+      newCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
+    }
+
+    if(newCenter.y() < m_selRect.height()/2) {
+      // set Y to the minimum possible Y
+      newCenter.setY(m_selRect.height()/2);
+    } else if(newCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
+      // set Y to the maximum possible Y
+      newCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
+    }
+
+    // move to the new center
+    m_selRect.moveCenter(newCenter);
+    // update the grab rectangle display
+    m_selRect.update();
     grabFrame();
   }
 }
@@ -582,16 +615,16 @@ void KMagZoomView::mouseMoveEvent(QMouseEvent *e)
 void KMagZoomView::keyPressEvent(QKeyEvent *e)
 {
   int offset = 16;
-  if (e->state() & QKeyEvent::ShiftButton)
+  if (e->state() & Qt::ShiftModifier)
     offset = 1;
 
-  if (e->key() == QKeyEvent::Key_Control)
+  if (e->key() == Qt::Key_Control)
     m_ctrlKeyPressed = true;
-  else if (e->key() == QKeyEvent::Key_Shift)
-    m_shiftKeyPressed = true;    
-  else if (e->key() == QKeyEvent::Key_Left)
+  else if (e->key() == Qt::Key_Shift)
+    m_shiftKeyPressed = true;
+  else if (e->key() == Qt::Key_Left)
   {
-    if (e->state() & QKeyEvent::ControlButton)
+    if (e->state() & Qt::ControlModifier)
     {
       if (offset >= m_selRect.width())
         m_selRect.setWidth (1);
@@ -611,14 +644,19 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
       if (offset > m_selRect.x())
         m_selRect.setX (0);
       else
-        m_selRect.moveBy (-offset,0);
+        m_selRect.translate (-offset,0);
     }
     m_selRect.update();
   }
-  else if (e->key() == QKeyEvent::Key_Right)
+  else if (e->key() == Qt::Key_Right)
   {
-    if (e->state() & QKeyEvent::ControlButton)
-      m_selRect.setRight (m_selRect.right()+offset);
+    if (e->state() & Qt::ControlModifier)
+    {
+      if (m_selRect.right()+offset >= QApplication::desktop()->width())
+        m_selRect.setRight (QApplication::desktop()->width()-1);
+      else
+        m_selRect.setRight (m_selRect.right()+offset);
+    }
     else if (contentsX() < contentsWidth()-visibleWidth())
     {
       offset = (int)(offset*m_zoom);
@@ -628,13 +666,17 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
         setContentsPos (contentsWidth()-visibleWidth(), contentsY());
     }
     else if (m_followMouse == false)
-      m_selRect.moveBy (offset,0);
-
+    {
+      if (m_selRect.right()+offset >= QApplication::desktop()->width())
+        m_selRect.moveTopRight (QPoint (QApplication::desktop()->width()-1, m_selRect.top()));
+      else
+        m_selRect.translate (offset,0);
+    }
     m_selRect.update();
   }
-  else if (e->key() == QKeyEvent::Key_Up)
+  else if (e->key() == Qt::Key_Up)
   {
-    if (e->state() & QKeyEvent::ControlButton)
+    if (e->state() & Qt::ControlModifier)
     {
       if (offset >= m_selRect.height())
         m_selRect.setHeight (1);
@@ -654,14 +696,19 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
       if (offset > m_selRect.y())
         m_selRect.setY (0);
       else
-        m_selRect.moveBy (0, -offset);
+        m_selRect.translate (0, -offset);
     }
     m_selRect.update();
   }
-  else if (e->key() == QKeyEvent::Key_Down)
+  else if (e->key() == Qt::Key_Down)
   {
-    if (e->state() & QKeyEvent::ControlButton)
-      m_selRect.setBottom (m_selRect.bottom()+offset);
+    if (e->state() & Qt::ControlModifier)
+    {
+      if (m_selRect.bottom()+offset >= QApplication::desktop()->height())
+        m_selRect.setBottom (QApplication::desktop()->height()-1);
+      else
+        m_selRect.setBottom (m_selRect.bottom()+offset);
+    }
     else if (contentsY() < contentsHeight()-visibleHeight())
     {
       offset = (int)(offset*m_zoom);
@@ -671,7 +718,12 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
         setContentsPos (contentsX(), contentsHeight()-visibleHeight());
     }
     else if (m_followMouse == false)
-      m_selRect.moveBy (0, offset);
+    {
+      if (m_selRect.bottom()+offset >= QApplication::desktop()->height())
+        m_selRect.moveBottomLeft (QPoint (m_selRect.left(), QApplication::desktop()->height()-1));
+      else
+        m_selRect.translate (0, offset);
+    }
     m_selRect.update();
   }
   else
@@ -680,23 +732,17 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
 
 void KMagZoomView::keyReleaseEvent(QKeyEvent *e)
 {
-  if (e->key() == QKeyEvent::Key_Control)
+  if (e->key() == Qt::Key_Control)
     m_ctrlKeyPressed = false;
-  else if (e->key() == QKeyEvent::Key_Shift)
+  else if (e->key() == Qt::Key_Shift)
     m_shiftKeyPressed = false;
   else
     e->ignore();
 }
 
-void KMagZoomView::contextMenuEvent (QContextMenuEvent *e)
-{
- emit contextMenu(e->globalPos());
- e->accept();
-}
-
 void KMagZoomView::focusOutEvent(QFocusEvent *e)
 {
-  if(e->lostFocus() == TRUE) {
+  if(e->lostFocus() == true) {
     m_ctrlKeyPressed = false;
     m_shiftKeyPressed = false;
   }
@@ -710,12 +756,44 @@ void KMagZoomView::focusOutEvent(QFocusEvent *e)
  */
 void KMagZoomView::fitToWindow()
 {
-  QPoint currCenter = m_selRect.center();
-  QRect newRect = m_invertedMatrix.mapRect (QRect(0, 0, visibleWidth(), visibleHeight()));
-  m_selRect.setSize (newRect.size());
-  m_selRect.moveCenter(currCenter);
-  m_selRect.update();
+  unsigned int newWidth, newHeight;
 
+  // this is a temporary solution, cast, maybe newWidth and newHeight should be float
+  if ((m_rotation == 90) || (m_rotation == 270))
+  {
+    newWidth = static_cast<unsigned int>(visibleHeight()/m_zoom);
+    newHeight = static_cast<unsigned int>(visibleWidth()/m_zoom);
+  } else {
+    newWidth = static_cast<unsigned int>(visibleWidth()/m_zoom);
+    newHeight = static_cast<unsigned int>(visibleHeight()/m_zoom);
+  }
+
+  QPoint currCenter = m_selRect.center();
+
+  m_selRect.setWidth(newWidth);
+  m_selRect.setHeight(newHeight);
+
+   // make sure the selection window does not go outside of the display
+   if(currCenter.x() < m_selRect.width()/2) {
+     // set X to the minimum possible X
+     currCenter.setX(m_selRect.width()/2);
+   } else if(currCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
+     // set X to the maximum possible X
+     currCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
+   }
+
+   if(currCenter.y() < m_selRect.height()/2) {
+     // set Y to the minimum possible Y
+     currCenter.setY(m_selRect.height()/2);
+   } else if(currCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
+     // set Y to the maximum possible Y
+     currCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
+   }
+
+  m_selRect.moveCenter(currCenter);
+  // update the grab rectangle display
+  m_selRect.update();
+//  m_fitToWindow = true;
   viewport()->repaint(false);
 }
 
@@ -742,42 +820,58 @@ void KMagZoomView::grabFrame()
      QPoint newCenter;
 
     // set new center to be the current mouse position
-    m_selRect.moveCenter(QCursor::pos());
+    newCenter = QCursor::pos();
+
+    // make sure the mouse position is not taking the grab window outside
+    // the display
+    if(newCenter.x() < m_selRect.width()/2) {
+      // set X to the minimum possible X
+      newCenter.setX(m_selRect.width()/2);
+    } else if(newCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
+      // set X to the maximum possible X
+      newCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
+    }
+
+    if(newCenter.y() < m_selRect.height()/2) {
+      // set Y to the minimum possible Y
+      newCenter.setY(m_selRect.height()/2);
+    } else if(newCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
+      // set Y to the maximum possible Y
+      newCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
+    }
+    // move to the new center
+    m_selRect.moveCenter(newCenter);
+
+    // update the grab rectangle display
     m_selRect.update();
   }
 
   //QRect r = pixmapRect();
 
   // define a normalized rectangle
-  QRect selRect = m_selRect.normalize();
+  QRect selRect = m_selRect.normalized();
 
   // grab screenshot from the screen and put it in the pixmap
   m_grabbedPixmap = QPixmap::grabWindow(QApplication::desktop()->winId(), selRect.x(), selRect.y(),
                                         selRect.width(), selRect.height());
 
-  // If the KMag window itself is in the screenshot, then it need to be filled with gray to avoid recursion
-  QPoint globalPos = viewport()->mapToGlobal (viewport()->rect().topLeft());
-  QRegion intersection (globalPos.x(), globalPos.y(), viewport()->width(), viewport()->height(), QRegion::Rectangle);
-  intersection &= QRegion (selRect, QRegion::Rectangle);
+  // colorize the grabbed pixmap, if applicable - TODO
+  if (m_colormode == 0)
+    m_coloredPixmap = m_grabbedPixmap;
+  else
+    m_coloredPixmap = ColorSim::recolor(m_grabbedPixmap, m_colormode);
 
-  // We don't want to overpaint other windows that happen to be on top
-  obscuredRegion (intersection, topLevelWidget()->winId(), m_selRect.winId());
-  intersection.translate (-selRect.x(), -selRect.y());
-
-  QPainter painter (&m_grabbedPixmap, true);
-  QMemArray<QRect> rects (intersection.rects());
-  for (uint i = 0; i < rects.size(); i++)
-    painter.fillRect (rects[i], QBrush (QColor (128, 128, 128)));
+  // zoom the image
+  m_zoomedPixmap = m_coloredPixmap.transformed(m_zoomMatrix);
 
   // call repaint to display the newly grabbed image
-  QRect newSize = m_zoomMatrix.mapRect (m_grabbedPixmap.rect());
-  resizeContents (newSize.width(), newSize.height());
+  resizeContents (m_zoomedPixmap.width(), m_zoomedPixmap.height());
   viewport()->repaint(false);
 }
 
 
 /**
- * Updates the mouse cursor in the zoom view
+ * Updates the mouse cursor in the zoom view.
  */
 void KMagZoomView::updateMouseView()
 {
@@ -809,8 +903,20 @@ void KMagZoomView::toggleRefresh()
  */
 void KMagZoomView::setZoom(float zoom)
 {
+  // use this zoom
   m_zoom = zoom;
-  updateMatrix();
+
+  // update selection window size when zooming in if necessary
+  if (m_fitToWindow)
+    fitToWindow();
+
+  // recompute the zoom matrix
+  m_zoomMatrix.reset();
+  m_zoomMatrix.scale(m_zoom, m_zoom);
+  m_zoomMatrix.rotate(m_rotation);
+
+  m_zoomedPixmap = m_coloredPixmap.transformed(m_zoomMatrix);
+
   viewport()->repaint();
 }
 
@@ -819,18 +925,32 @@ void KMagZoomView::setZoom(float zoom)
  */
 void KMagZoomView::setRotation(int rotation)
 {
+  // use this rotation
   m_rotation = rotation;
-  updateMatrix();
+
+  // update selection window size if necessary
+  if (m_fitToWindow)
+    fitToWindow();
+
+  // recompute the zoom matrix
+  m_zoomMatrix.reset();
+  m_zoomMatrix.scale(m_zoom, m_zoom);
+  m_zoomMatrix.rotate(m_rotation);
+
+  m_zoomedPixmap = m_coloredPixmap.transformed(m_zoomMatrix);
+
   viewport()->repaint();
 }
 
 /**
- * This function sets whether the magnified image is shown inverted
+ * Set a new color simulation mode.
  */
-void KMagZoomView::setInvertation(bool invert)
+void KMagZoomView::setColorMode(int mode)
 {
-  m_invert = invert;
-  viewport()->repaint();
+  if (m_colormode != mode) {
+    m_colormode = mode;
+    viewport()->repaint();
+  }
 }
 
 /**
@@ -840,10 +960,10 @@ void KMagZoomView::setRefreshRate(float fps)
 {
   if(fps < 0.1)
     return;
-  m_fps = static_cast<unsigned int>(fps);  
-  
+  m_fps = static_cast<unsigned int>(fps);
+
   if(m_grabTimer.isActive())
-    m_grabTimer.changeInterval(static_cast<int>(1000.0/m_fps));
+    m_grabTimer.start(static_cast<int>(1000.0/m_fps));
 }
 
 void KMagZoomView::showSelRect(bool show)
@@ -888,40 +1008,21 @@ QStringList KMagZoomView::getShowMouseStringList() const
 
 
 /**
- * Returns the image which is being displayed. Its again drawn by adding
+ * Returns the image which is being displayed. It's again drawn by adding
  * the mouse cursor if needed.
  */
 QPixmap KMagZoomView::getPixmap()
 {
   // show the pixel under mouse cursor
-  if(m_showMouse && !m_grabbedPixmap.isNull()) {
-    // Pixmap which will have the pixmap + mouse
-    QPixmap mousePixmap(m_grabbedPixmap);
+  if(m_showMouse && !m_zoomedPixmap.isNull()) {
+    // Pixmap which will have the zoomed pixmap + mouse
+    QPixmap zoomView(m_zoomedPixmap);
 
     // paint the mouse cursor w/o updating to a newer position
-    paintMouseCursor(&mousePixmap, calcMousePos(false));
-    
-    return(mousePixmap);
+    paintMouseCursor(&zoomView, calcMousePos(false));
+
+    return(zoomView);
   } else { // no mouse cursor
-     return(m_grabbedPixmap);
+     return(m_zoomedPixmap);
   }
 }
-
-/**
- * Update the magnification matrix
- */
-void KMagZoomView::updateMatrix()
-{
-  // update selection window size if necessary
-  if (m_fitToWindow)
-    fitToWindow();
-
-  // recompute the zoom matrix
-  m_zoomMatrix.reset();
-  m_zoomMatrix.scale(m_zoom, m_zoom);
-  m_zoomMatrix.rotate(m_rotation);
-
-  bool inverted;
-  m_invertedMatrix = m_zoomMatrix.invert (&inverted);
-
-}
