@@ -11,59 +11,24 @@
 //
 
 // QT includes
-// #include <qtextcodec.h>
-#include <qstringlist.h>
+#include <QtCore/QTextCodec>
 
 // KDE includes
 #include <kdebug.h>
+#include <kglobal.h>
 #include <klocale.h>
+#include <kstandarddirs.h>
 #include <kdemacros.h>
-#include <arts/artsmodules.h>
+#include "kdeexportfix.h"
 
 // App specific includes
 #include "ksayitfreeverblib.h"
 #include "freeverbsetupimpl.h"
+#include <arts/artsmodules.h>
 
-
-// The class factory
-K_EXPORT_COMPONENT_FACTORY( libFreeverb_plugin, FreeverbPluginFactory )
-/** replaced by macro
-extern "C"
-{
-    void* init_libFreeverb_plugin()
-    {
-        return new FreeverbPluginFactory;    
-    }
-};
-*/
-
-KInstance *FreeverbPluginFactory::p_instance = 0L;
-
-
-// Factory Constructor
-FreeverbPluginFactory::FreeverbPluginFactory(QObject *parent, const char* name)
-{
-    p_instance = new KInstance("FreeverbPluginFactory");
-}
-
-    
-QObject* FreeverbPluginFactory::createObject(QObject *parent, const char* name, 
-            const char*, 
-            const QStringList &)
-{
-    kdDebug(100200) << "FreeverbPluginFactory::createObject()" << endl;
-    
-    QObject* obj = new FreeverbPlugin( parent, name );
-    emit objectCreated( obj );
-    return obj;  
-}        
-
-
-
-
-// Plugin Constructor
-FreeverbPlugin::FreeverbPlugin(QObject *parent, const char* name) //, KApplication *Appl)
- : FXPlugin(parent, name) //, m_Appl(Appl)
+// Constructor
+FreeverbPlugin::FreeverbPlugin(QObject *parent, const char* name, KApplication *Appl)
+ : QObject(parent, name), FXPlugin(), m_Appl(Appl)
 {
     m_config = new KSimpleConfig("ksayit_freeverbrc");
     
@@ -74,14 +39,6 @@ FreeverbPlugin::~FreeverbPlugin()
     delete m_config;
 }
 
-/** sets the Main application object
-*/
-void FreeverbPlugin::setApplication(KApplication *Appl)
-{
-    m_Appl = Appl;
-}
-
-
 /** returns the Name of the Plugin
 */
 QString FreeverbPlugin::getName_KS() const
@@ -116,10 +73,10 @@ bool FreeverbPlugin::showGUI_KS()
 long FreeverbPlugin::activate_KS(KArtsServer *server,
                             StereoEffectStack *fx_stack) const
 {
-    // kdDebug(100200) << "Aktiviere Synth_FREEVERB-Effekt" << endl;
+    // kDebug(100200) << "Aktiviere Synth_FREEVERB-Effekt";
     Synth_FREEVERB fv = DynamicCast( server->server().createObject("Arts::Synth_FREEVERB") );
     if ( !fv.isNull() ){
-        // kdDebug(100200) << "Filter angelegt" << endl;
+        // kDebug(100200) << "Filter angelegt";
         // get filter parameter
         m_config->setGroup("Synth_FREEVERB");            
         fv.roomsize( m_config->readDoubleNumEntry("roomsize", 50.0)/100.0 );
@@ -139,7 +96,7 @@ long FreeverbPlugin::activate_KS(KArtsServer *server,
 bool FreeverbPlugin::deactivate_KS(StereoEffectStack *fx_stack,
                             long EffectID ) const
 {
-    // kdDebug(100200) << "Deaktiviere Synth_FREEVERB-Effekt" << endl;
+    // kDebug(100200) << "Deaktiviere Synth_FREEVERB-Effekt";
     if (EffectID!=0){
         fx_stack->remove(EffectID);
     }
@@ -147,7 +104,18 @@ bool FreeverbPlugin::deactivate_KS(StereoEffectStack *fx_stack,
 }
 
 
-
+// The class factories
+extern "C"
+{
+    KDE_EXPORT FXPlugin* createPlugin(KApplication *Appl)
+    {
+        return new FreeverbPlugin(0, "freeverb_plugin", Appl);
+    }
+    KDE_EXPORT void destroyPlugin(FXPlugin* p)
+    {
+        delete p;
+    }
+};
 
 
 
